@@ -12,24 +12,23 @@ class PostTest(TestCase):
     def _test(self, name, any_id_ok=False):
         with open(os.path.join(os.path.dirname(__file__), 'data', '{name}.xml'.format(name=name))) as f:
             instance = f.read()
-        doc_id = create_xform_from_xml(instance)
-        xform = XFormInstance.get(doc_id)
-
-        try:
-            xform_json = xform.to_json()
-            with open(os.path.join(os.path.dirname(__file__), 'data', '{name}.json'.format(name=name))) as f:
-                result = json.load(f)
-            result['received_on'] = xform_json['received_on']
-            result['_rev'] = xform_json['_rev']
-            if any_id_ok:
-                result['_id'] = xform_json['_id']
-            self.assertDictEqual(xform_json, result)
-        except Exception:
-            # to help when bootstrapping a new test case
-            print json.dumps(xform_json)
-            raise
-        finally:
-            xform.delete()
+        with create_xform_from_xml(instance) as doc_id:
+            xform = XFormInstance.get(doc_id)
+            try:
+                xform_json = xform.to_json()
+                with open(os.path.join(os.path.dirname(__file__), 'data', '{name}.json'.format(name=name))) as f:
+                    result = json.load(f)
+                result['received_on'] = xform_json['received_on']
+                result['_rev'] = xform_json['_rev']
+                if any_id_ok:
+                    result['_id'] = xform_json['_id']
+                self.assertDictEqual(xform_json, result)
+            except Exception:
+                # to help when bootstrapping a new test case
+                print json.dumps(xform_json)
+                raise
+            finally:
+                xform.delete()
 
     def test_cloudant_template(self):
         self._test('cloudant-template')
