@@ -4,6 +4,7 @@ import datetime
 import logging
 
 from StringIO import StringIO
+from django.conf import settings
 from django.test.client import Client
 
 from couchdbkit import ResourceConflict, ResourceNotFound, resource
@@ -15,11 +16,14 @@ from django.http import (
 )
 from redis import ConnectionError
 
-import couchforms
-from couchforms.const import MAGIC_PROPERTY, MULTIPART_FILENAME_ERROR
-from couchforms.exceptions import DuplicateError
 from dimagi.utils.mixins import UnicodeMixIn
-from couchforms.models import (
+from dimagi.utils.couch import uid, LockManager, ReleaseOnError, release_lock
+import xml2json
+
+import couchforms
+from . import const
+from .exceptions import DuplicateError
+from .models import (
     DefaultAuthContext,
     SubmissionErrorLog,
     XFormDeprecated,
@@ -28,13 +32,11 @@ from couchforms.models import (
     XFormInstance,
     doc_types,
 )
-from couchforms.signals import (
+from .signals import (
     ReceiverResult,
     successful_form_received,
 )
-from dimagi.utils.couch import uid, LockManager, ReleaseOnError, release_lock
-from couchforms.xml import ResponseNature, OpenRosaResponse
-import xml2json
+from .xml import ResponseNature, OpenRosaResponse
 
 
 class SubmissionError(Exception, UnicodeMixIn):
@@ -88,7 +90,8 @@ def convert_xform_to_json(xml_string):
 
 
 def acquire_lock_for_xform(xform_id):
-    lock = XFormInstance.get_obj_lock_by_id(xform_id)
+    # this is high, but I want to test if MVP conflicts disappear
+    lock = XFormInstance.get_obj_lock_by_id(xform_id, timeout_seconds=2*60)
     try:
         lock.acquire()
     except ConnectionError:
@@ -154,20 +157,23 @@ def create_xform_from_xml(xml_string, _id=None, process=None):
     return LockManager(xform.get_id, lock)
 
 
-def post_xform_to_couch(instance, attachments=None, process=None):
+def post_xform_to_couch(instance, attachments=None, process=None,
+                        domain='test-domain'):
     """
     create a new xform and releases the lock
 
-    good for testing but not to be used in active code
+    this is a testing entry point only and is not to be used in real code
 
     """
+    assert getattr(settings, 'UNIT_TESTING', False)
     xform_lock = create_and_lock_xform(instance, attachments=attachments,
-                                       process=process)
+                                       process=process, domain=domain)
     with xform_lock as xform:
         return xform
 
 
-def create_and_lock_xform(instance, attachments=None, process=None):
+def create_and_lock_xform(instance, attachments=None, process=None,
+                          domain=None, _id=None):
     """
     Save a new xform to couchdb in a thread-safe manner
     Returns a LockManager containing the new XFormInstance and its lock,
@@ -180,12 +186,14 @@ def create_and_lock_xform(instance, attachments=None, process=None):
     attachments = attachments or {}
 
     try:
-        doc_id, lock = create_xform_from_xml(instance, process=process)
+        doc_id, lock = create_xform_from_xml(instance, process=process,
+                                             _id=_id)
     except couchforms.XMLSyntaxError as e:
         doc = _log_hard_failure(instance, attachments, e)
         raise SubmissionError(doc)
     except DuplicateError:
-        return _handle_id_conflict(instance, attachments, process=process)
+        return _handle_id_conflict(instance, attachments, process=process,
+                                   domain=domain)
 
     try:
         xform = XFormInstance.get(doc_id)
@@ -218,7 +226,7 @@ def _extract_id_from_raw_xml(xml):
     return _extract_meta_instance_id(json_form) or ''
 
 
-def _handle_id_conflict(instance, attachments, process):
+def _handle_id_conflict(instance, attachments, process, domain):
     """
     For id conflicts, we check if the files contain exactly the same content,
     If they do, we just log this as a dupe. If they don't, we deprecate the
@@ -228,8 +236,26 @@ def _handle_id_conflict(instance, attachments, process):
     conflict_id = _extract_id_from_raw_xml(instance)
 
     # get old document
-    existing_doc = XFormInstance.get(conflict_id)
+    existing_doc = XFormInstance.get_db().get(conflict_id)
+    assert domain
+    if existing_doc.get('domain') != domain\
+            or existing_doc.get('doc_type') not in doc_types():
+        # exit early
+        return create_and_lock_xform(instance, attachments=attachments,
+                                     process=process, _id=uid.new())
+    else:
+        existing_doc = XFormInstance.wrap(existing_doc)
+        return _handle_duplicate(existing_doc, instance, attachments, process)
 
+
+def _handle_duplicate(existing_doc, instance, attachments, process):
+    """
+    Handle duplicate xforms and xform editing ('deprecation')
+
+    existing doc *must* be validated as an XFormInstance in the right domain
+
+    """
+    conflict_id = existing_doc.get_id
     # compare md5s
     existing_md5 = existing_doc.xml_md5()
     new_md5 = hashlib.md5(instance).hexdigest()
@@ -331,14 +357,9 @@ def scrub_meta(xform):
 class SubmissionPost(object):
 
     failed_auth_response = HttpResponseForbidden('Bad auth')
-    bad_multipart_response = HttpResponseBadRequest((
-        'If you use multipart/form-data, please name your file %s.\n'
-        'You may also do a normal (non-multipart) post '
-        'with the xml submission as the request body instead.'
-    ) % MAGIC_PROPERTY)
 
-    def __init__(self, instance=None, attachments=None,
-                 auth_context=None, domain=None, app_id=None, path=None,
+    def __init__(self, instance=None, attachments=None, auth_context=None,
+                 domain=None, app_id=None, build_id=None, path=None,
                  location=None, submit_ip=None, openrosa_headers=None,
                  last_sync_token=None, received_on=None, date_header=None):
         assert domain, domain
@@ -347,6 +368,7 @@ class SubmissionPost(object):
         # get_location has good default
         self.domain = domain
         self.app_id = app_id
+        self.build_id = build_id
         self.location = location or couchforms.get_location()
         self.received_on = received_on
         self.date_header = date_header
@@ -377,6 +399,7 @@ class SubmissionPost(object):
 
         doc['domain'] = self.domain
         doc['app_id'] = self.app_id
+        doc['build_id'] = self.build_id
         doc['#export_tag'] = ["domain", "xmlns"]
 
         return doc
@@ -385,8 +408,8 @@ class SubmissionPost(object):
         if not self.auth_context.is_valid():
             return self.failed_auth_response
 
-        if self.instance is MULTIPART_FILENAME_ERROR:
-            return self.bad_multipart_response
+        if isinstance(self.instance, const.BadRequest):
+            return HttpResponseBadRequest(self.instance.message)
 
         def process(xform):
             self._attach_shared_props(xform)
